searchData={"items":[{"type":"module","title":"HashRing","doc":"This module defines an API for creating/manipulating a hash ring.\nThe internal data structure for the hash ring is actually a gb_tree, which provides\nfast lookups for a given key on the ring.\n\n- The ring is a continuum of 2^32 \"points\", or integer values.\n- Nodes are sharded into 128 points, and distributed across the ring.\n- Each shard owns the keyspace below it.\n- Keys are hashed and assigned a point on the ring, the node for a given\n  ring is determined by finding the next highest point on the ring for a shard,\n  the node that shard belongs to is then the node which owns that key.\n- If a key's hash does not have any shards above it, it belongs to the first shard,\n  this mechanism is what creates the ring-like topology.\n- When nodes are added/removed from the ring, only a small subset of keys must be reassigned.","ref":"HashRing.html"},{"type":"function","title":"HashRing.add_node/3","doc":"Adds a node to the hash ring, with an optional weight provided which\ndetermines the number of virtual nodes (shards) that will be assigned to\nit on the ring.\n\nThe default weight for a node is `128`.","ref":"HashRing.html#add_node/3"},{"type":"function","title":"Examples - HashRing.add_node/3","doc":"iex> ring = HashRing.new()\n    ...> ring = HashRing.add_node(ring, \"a\")\n    ...> %HashRing{nodes: [\"b\", \"a\"]} = ring = HashRing.add_node(ring, \"b\", 64)\n    ...> HashRing.key_to_node(ring, :foo)\n    \"a\"","ref":"HashRing.html#add_node/3-examples"},{"type":"function","title":"HashRing.add_nodes/2","doc":"Adds a list of nodes to the hash ring.\n\nThe list can contain just the node key, or a tuple of the node key and its desired weight.\n\nSee also the documentation for `add_node/3`.","ref":"HashRing.html#add_nodes/2"},{"type":"function","title":"Examples - HashRing.add_nodes/2","doc":"iex> ring = HashRing.new()\n    ...> ring = HashRing.add_nodes(ring, [\"a\", {\"b\", 64}])\n    ...> %HashRing{nodes: [\"b\", \"a\"]} = ring\n    ...> HashRing.key_to_node(ring, :foo)\n    \"a\"","ref":"HashRing.html#add_nodes/2-examples"},{"type":"function","title":"HashRing.key_to_node/2","doc":"Determines which node owns the given key.\n\nThis function assumes that the ring has been populated with at least one node.","ref":"HashRing.html#key_to_node/2"},{"type":"function","title":"Examples - HashRing.key_to_node/2","doc":"iex> ring = HashRing.new(\"a\")\n    ...> HashRing.key_to_node(ring, :foo)\n    \"a\"\n\n    iex> ring = HashRing.new()\n    ...> HashRing.key_to_node(ring, :foo)\n    {:error, {:invalid_ring, :no_nodes}}","ref":"HashRing.html#key_to_node/2-examples"},{"type":"function","title":"HashRing.key_to_nodes/3","doc":"Determines which nodes owns a given key. Will return either `count` results or\nthe number of nodes, depending on which is smaller.\n\nThis function assumes that the ring has been populated with at least one node.","ref":"HashRing.html#key_to_nodes/3"},{"type":"function","title":"Examples - HashRing.key_to_nodes/3","doc":"iex> ring = HashRing.new()\n    ...> ring = HashRing.add_node(ring, \"a\")\n    ...> ring = HashRing.add_node(ring, \"b\")\n    ...> ring = HashRing.add_node(ring, \"c\")\n    ...> HashRing.key_to_nodes(ring, :foo, 2)\n    [\"b\", \"c\"]\n\n    iex> ring = HashRing.new()\n    ...> HashRing.key_to_nodes(ring, :foo, 1)\n    {:error, {:invalid_ring, :no_nodes}}","ref":"HashRing.html#key_to_nodes/3-examples"},{"type":"function","title":"HashRing.new/0","doc":"Creates a new hash ring structure, with no nodes added yet.","ref":"HashRing.html#new/0"},{"type":"function","title":"Examples - HashRing.new/0","doc":"iex> ring = HashRing.new()\n    ...> %HashRing{nodes: [\"a\"]} = ring = HashRing.add_node(ring, \"a\")\n    ...> HashRing.key_to_node(ring, {:complex, \"key\"})\n    \"a\"","ref":"HashRing.html#new/0-examples"},{"type":"function","title":"HashRing.new/2","doc":"Creates a new hash ring structure, seeded with the given node,\nwith an optional weight provided which determines the number of\nvirtual nodes (shards) that will be assigned to it on the ring.\n\nThe default weight for a node is `128`.","ref":"HashRing.html#new/2"},{"type":"function","title":"Examples - HashRing.new/2","doc":"iex> ring = HashRing.new(\"a\")\n    ...> %HashRing{nodes: [\"a\"]} = ring\n    ...> HashRing.key_to_node(ring, :foo)\n    \"a\"\n\n    iex> ring = HashRing.new(\"a\", 200)\n    ...> %HashRing{nodes: [\"a\"]} = ring\n    ...> HashRing.key_to_node(ring, :foo)\n    \"a\"","ref":"HashRing.html#new/2-examples"},{"type":"function","title":"HashRing.nodes/1","doc":"Returns the list of nodes which are present on the ring.\n\nThe type of the elements in this list are the same as the type of the elements\nyou initially added to the ring. In the following example, we used strings, but\nif you were using atoms, such as those used for Erlang node names, you would get\na list of atoms back.\n\n    iex> ring = HashRing.new |> HashRing.add_nodes([\"a\", \"b\"])\n    ...> HashRing.nodes(ring)\n    [\"b\", \"a\"]","ref":"HashRing.html#nodes/1"},{"type":"function","title":"HashRing.remove_node/2","doc":"Removes a node from the hash ring.","ref":"HashRing.html#remove_node/2"},{"type":"function","title":"Examples - HashRing.remove_node/2","doc":"iex> ring = HashRing.new()\n    ...> %HashRing{nodes: [\"a\"]} = ring = HashRing.add_node(ring, \"a\")\n    ...> %HashRing{nodes: []} = ring = HashRing.remove_node(ring, \"a\")\n    ...> HashRing.key_to_node(ring, :foo)\n    {:error, {:invalid_ring, :no_nodes}}","ref":"HashRing.html#remove_node/2-examples"},{"type":"type","title":"HashRing.t/0","doc":"","ref":"HashRing.html#t:t/0"},{"type":"module","title":"HashRing.Managed","doc":"This module defines the API for working with hash rings where the ring state is managed\nin a GenServer process.\n\nThere is a performance penalty with working with the ring this way, but it is the best approach\nif you need to share the ring across multiple processes, or need to maintain multiple rings.\n\nIf your rings map 1:1 with Erlang node membership, you can configure rings to automatically\nmonitor node up/down events and update the hash ring accordingly, with a default weight,\nand either whitelist or blacklist nodes from the ring. You configure this at the ring level in your `config.exs`.\n\nEach ring is configured in `config.exs`, and can contain a list of nodes to seed the ring with,\nand you can then dynamically add/remove nodes to the ring using the API here. Each node on the ring can\nbe configured with a weight, which affects the amount of the total keyspace it owns. The default weight\nis `128`. It's best to base the weight of nodes on some concrete relative value, such as the amount of\nmemory a node has.","ref":"HashRing.Managed.html"},{"type":"function","title":"HashRing.Managed.add_node/2","doc":"Adds a node to the given hash ring.\n\nAn error is returned if the ring does not exist, or the node already exists in the ring.","ref":"HashRing.Managed.html#add_node/2"},{"type":"function","title":"Examples - HashRing.Managed.add_node/2","doc":"iex> {:ok, _pid} = HashRing.Managed.new(:test4)\n    ...> HashRing.Managed.add_node(:test4, \"a\")\n    ...> HashRing.Managed.key_to_node(:test4, :foo)\n    \"a\"\n\n    iex> HashRing.Managed.add_node(:no_exist, \"a\")\n    {:error, :no_such_ring}","ref":"HashRing.Managed.html#add_node/2-examples"},{"type":"function","title":"HashRing.Managed.add_node/3","doc":"Same as `add_node/2`, but takes a weight value.\n\nThe weight controls the relative presence this node will have on the ring,\nthe default is `128`, but it's best to give each node a weight value which maps\nto a concrete resource such as memory or priority. It's not ideal to have a number\nwhich is too high, as it will make the ring data structure larger, but a good value\nis probably in the range of 64-256.","ref":"HashRing.Managed.html#add_node/3"},{"type":"function","title":"Examples - HashRing.Managed.add_node/3","doc":"iex> {:ok, _pid} = HashRing.Managed.new(:test5)\n    ...> HashRing.Managed.add_node(:test5, \"a\", 64)\n    ...> HashRing.Managed.key_to_node(:test5, :foo)\n    \"a\"\n\n    iex> HashRing.Managed.add_node(:no_exist, \"a\")\n    {:error, :no_such_ring}","ref":"HashRing.Managed.html#add_node/3-examples"},{"type":"function","title":"HashRing.Managed.add_nodes/2","doc":"Adds a list of nodes to the ring.\n\nThe list of nodes can contain either node names or `{node_name, weight}`\ntuples. If there is an error with any of the node weights, an error will\nbe returned, and the ring will remain unchanged.","ref":"HashRing.Managed.html#add_nodes/2"},{"type":"function","title":"Examples - HashRing.Managed.add_nodes/2","doc":"iex> {:ok, _pid} = HashRing.Managed.new(:test6)\n    ...> :ok = HashRing.Managed.add_nodes(:test6, [\"a\", {\"b\", 64}])\n    ...> HashRing.Managed.key_to_node(:test6, :foo)\n    \"a\"\n\n    iex> {:ok, _pid} = HashRing.Managed.new(:test7)\n    ...> HashRing.Managed.add_nodes(:test7, [\"a\", {\"b\", :wrong}])\n    {:error, [{:invalid_weight, \"b\", :wrong}]}","ref":"HashRing.Managed.html#add_nodes/2-examples"},{"type":"function","title":"HashRing.Managed.child_spec/1","doc":"","ref":"HashRing.Managed.html#child_spec/1"},{"type":"function","title":"HashRing.Managed.key_to_node/2","doc":"Maps a key to a node on the hash ring.\n\nAn error is returned if the ring does not exist.","ref":"HashRing.Managed.html#key_to_node/2"},{"type":"function","title":"HashRing.Managed.key_to_nodes/3","doc":"Maps a key to a set of nodes on the hash ring.\n\nAn error is returned if the ring does not exist.","ref":"HashRing.Managed.html#key_to_nodes/3"},{"type":"function","title":"HashRing.Managed.new/2","doc":"Creates a new stateful hash ring with the given name.\n\nThis name is how you will refer to the hash ring via other API calls.\n\nIt takes an optional set of options which control how the ring behaves.\nValid options are as follows:\n\n* `nodes: list` - a list of nodes to initialize the ring.\n* `monitor_nodes: boolean` - will automatically monitor Erlang node membership,\n  if new nodes are connected or nodes are disconnected, the ring will be updated automatically.\n  In this configuration, nodes cannot be added or removed via the API. Those requests will be ignored.\n* `node_blacklist: [String.t | Regex.t]` - Used in conjunction with `monitor_nodes: true`, this\n  is a list of patterns, either as literal strings, or as regex patterns (in either string or literal form),\n  and will be used to ignore nodeup/down events for nodes which are blacklisted. If a node whitelist\n  is provided, the blacklist has no effect.\n* `node_whitelist: [String.t | Regex.t]` - The same as `node_blacklist`, except the opposite; only nodes\n  which match a pattern in the whitelist will result in the ring being updated.\n* `node_type: :all | :hidden | :visible`: refers what kind of nodes will be monitored\n  when `monitor_nodes` is `true`. For more information, see `:net_kernel.monitor_nodes/2`.\n\nAn error is returned if the ring already exists or if bad ring options are provided.","ref":"HashRing.Managed.html#new/2"},{"type":"function","title":"Examples - HashRing.Managed.new/2","doc":"iex> {:ok, _pid} = HashRing.Managed.new(:test1, [nodes: [\"a\", {\"b\", 64}]])\n    ...> HashRing.Managed.key_to_node(:test1, :foo)\n    \"a\"\n\n    iex> {:ok, pid} = HashRing.Managed.new(:test2)\n    ...> {:error, {:already_started, existing_pid}} = HashRing.Managed.new(:test2)\n    ...> pid == existing_pid\n    true\n    iex> HashRing.Managed.new(:test3, [nodes: \"a\"])\n    ** (ArgumentError) {:nodes, \"a\"} is an invalid option for `HashRing.Managed.new/2`","ref":"HashRing.Managed.html#new/2-examples"},{"type":"function","title":"HashRing.Managed.nodes/1","doc":"Same as `HashRing.nodes/1`, returns a list of nodes on the ring.","ref":"HashRing.Managed.html#nodes/1"},{"type":"function","title":"Examples - HashRing.Managed.nodes/1","doc":"iex> {:ok, _pid} = HashRing.Managed.new(:nodes_test)\n    ...> HashRing.Managed.add_nodes(:nodes_test, [:a, :b])\n    ...> HashRing.Managed.nodes(:nodes_test)\n    [:b, :a]","ref":"HashRing.Managed.html#nodes/1-examples"},{"type":"function","title":"HashRing.Managed.remove_node/2","doc":"Removes a node from the given hash ring.\n\nAn error is returned if the ring does not exist.","ref":"HashRing.Managed.html#remove_node/2"},{"type":"function","title":"Examples - HashRing.Managed.remove_node/2","doc":"iex> {:ok, _pid} = HashRing.Managed.new(:test8)\n    ...> :ok = HashRing.Managed.add_nodes(:test8, [\"a\", {\"b\", 64}])\n    ...> :ok = HashRing.Managed.remove_node(:test8, \"b\")\n    ...> HashRing.Managed.key_to_node(:test8, :foo)\n    \"a\"","ref":"HashRing.Managed.html#remove_node/2-examples"},{"type":"type","title":"HashRing.Managed.child_spec_option/0","doc":"","ref":"HashRing.Managed.html#t:child_spec_option/0"},{"type":"type","title":"HashRing.Managed.child_spec_options/0","doc":"","ref":"HashRing.Managed.html#t:child_spec_options/0"},{"type":"type","title":"HashRing.Managed.key/0","doc":"","ref":"HashRing.Managed.html#t:key/0"},{"type":"type","title":"HashRing.Managed.node_list/0","doc":"","ref":"HashRing.Managed.html#t:node_list/0"},{"type":"type","title":"HashRing.Managed.pattern_list/0","doc":"","ref":"HashRing.Managed.html#t:pattern_list/0"},{"type":"type","title":"HashRing.Managed.ring/0","doc":"","ref":"HashRing.Managed.html#t:ring/0"},{"type":"type","title":"HashRing.Managed.ring_options/0","doc":"","ref":"HashRing.Managed.html#t:ring_options/0"},{"type":"type","title":"HashRing.Managed.weight/0","doc":"","ref":"HashRing.Managed.html#t:weight/0"},{"type":"extras","title":"Overview","doc":"# libring - A fast consistent hash ring for Elixir\n\n[![Module Version](https://img.shields.io/hexpm/v/libring.svg)](https://hex.pm/packages/libring)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/libring/)\n[![Total Download](https://img.shields.io/hexpm/dt/libring.svg)](https://hex.pm/packages/libring)\n[![License](https://img.shields.io/hexpm/l/libring.svg)](https://github.com/bitwalker/libring/blob/master/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/bitwalker/libring.svg)](https://github.com/bitwalker/libring/commits/master)\n\nThis library implements a stateful consistent hash ring. It's extremely fast\n(in benchmarks it's faster than all other implementations I've tested against,\nnamely [voicelayer/hash-ring](https://github.com/voicelayer/hash-ring) and\n[sile/hash_ring](https://github.com/sile/hash_ring)), it has no external dependencies,\nand is written in Elixir.\n\nThe algorithm is based on [libketama](https://github.com/rj/ketama). Nodes on the\nring are broken into shards and each one is assigned an integer value in the keyspace, which\nis the set of integers from 1 to 2^32-1. The distribution of these shards is random, but\ndeterministic.\n\nKeys are then mapped to a shard by converting the key to a binary, hashing it with SHA-256,\nconverting the hash to an integer in the keyspace, then finding the shard which is assigned\nthe next highest value, if there is no next highest value, the lowest integer is used, which\nis how the \"ring\" is formed.\n\nThis implementation uses a general balanced tree, via Erlang's `:gb_tree` module. Each shard\nis inserted into the tree, and we use this data structure to efficiently lookup next-highest\nkey and smallest key. I suspect this is why `libring` is faster than other implementations I've\nbenchmarked against.","ref":"readme.html"},{"type":"extras","title":"Usage - Overview","doc":"Add `:libring` to your deps, and run `mix deps.get`.\n\n```elixir\ndef deps do\n  [\n    {:libring, \"~> 1.0\"}\n  ]\nend\n```\n\nYou have two choices for managing hash rings in your application:\n\n### `HashRing`\n\nThis API works with the raw ring data structure. It is the fastest implementation,\nand is best suited for when you have a single process which will need to access the\nring, and which can hold the ring in its internal state.\n\n```elixir\nring = HashRing.new()\n       |> HashRing.add_node(\"a\")\n       |> HashRing.add_node(\"b\")\n\n\"a\" = HashRing.key_to_node(ring, {:myworker, 123})\n```\n\nYou can also specify the weight of each node, and add nodes in bulk:\n\n```elixir\nring = HashRing.new()\n       |> HashRing.add_nodes([\"a\", {\"b\", 64}])\n       |> HashRing.add_node(\"c\", 200)\n\"c\" = HashRing.key_to_node(ring, {:myworker, 123})\n```\n\n**NOTE**: Node names do not have to be strings, they can be atoms, tuples, etc.\n\n### `HashRing.Managed`\n\nThis API works with rings which are held in the internal state of a GenServer process.\nIt supports the same API as `HashRing`. Because of this, there is a performance overhead\ndue to the messaging, and the GenServer can be a potential bottleneck. If this is the case\nyou are better off exploring ways to use the raw `HashRing` API. However this API is best suited\nfor situations where you have multiple processes accessing the ring, or need to maintain multiple\nrings.\n\n**NOTE**: You must have the `:libring` application started to use this API.\n\n```elixir\n{:ok, pid} = HashRing.Managed.new(:myring)\n:ok = HashRing.Managed.add_node(:myring, \"a\")\n:ok = HashRing.Managed.add_node(:myring, \"b\", 64)\n:ok = HashRing.Managed.add_node(:myring, \"c\", 200)\n\"c\" = HashRing.Managed.key_to_node(:myring, {:myworker, 123})\n```\n\nYou can configure managed rings in `config.exs`, and they will be created and initialized\nwhen the `:libring` application starts. Configured rings take two shapes, static and dynamic\nrings. Static rings are simply those where the nodes are provided up front, although you can\nalways add/remove nodes manually at runtime; dynamic rings have Erlang node monitoring enabled,\nand add or remove nodes on the ring based on cluster membership.\n\nYou can whitelist/blacklist nodes when using dynamic rings, so that only those nodes which you\nactually want to distribute work to are used in calculations. This configuration is shown below as well.\n\nIf you provide a whitelist, the blacklist will have no effect, and only nodes matching the whitelist\nwill be added. If you do not provide a whitelist, the blacklist will be used to filter nodes. If you\ndo not provide either, a default blacklist containing the `~r/^remsh.*$/` pattern from the example below,\nwhich is a good default to prevent remote shell sessions (at least those done via releases) from causing\nthe ring to change.\n\nThe whitelist and blacklist only have an effect when `monitor_nodes: true`.","ref":"readme.html#usage"},{"type":"extras","title":"Configuration - Overview","doc":"Below is an example configuration:\n\n```elixir\nconfig :libring,\n  rings: [\n    # A ring which automatically changes based on Erlang cluster membership,\n    # but does not allow nodes named \"a\" or \"remsh*\" to be added to the ring\n    ring_a: [monitor_nodes: true,\n             node_type: :visible,\n             node_blacklist: [\"a\", ~r/^remsh.*$/]],\n    # A ring which is composed of three nodes, of which \"c\" has a non-default weight of 200\n    # The default weight is 128\n    ring_b: [nodes: [\"a\", \"b\", {\"c\", 200}]]\n  ]\n```","ref":"readme.html#configuration"},{"type":"extras","title":"Contributing - Overview","doc":"If you have changes in mind that are significant or potentially time-consuming, please open an RFC-style PR first, where we\ncan discuss your plans first. I don't want you to spend all your time crafting a PR that I ultimately reject because I don't\nthink it's a good fit or is too large for me to review. Not that I plan to reject PRs in general, but I have to be careful to\nbalance features with maintenance burden, or I will quickly be unable to manage the project.\n\nPlease ensure that you adhere to a commit style where logically related changes are in a single commit, or broken up in a way that\neases review if necessary. Keep commit subject lines informative, but short, and provide additional detail in the extended message text\nif needed. If you can, mention relevant issue numbers in either the subject or the extended message.","ref":"readme.html#contributing"},{"type":"extras","title":"Copyright and License - Overview","doc":"Copyright (c) 2016 Paul Schoenfelder\n\nThis library is MIT licensed. See the\n[LICENSE](https://github.com/bitwalker/libring/blob/master/LICENSE) for details.","ref":"readme.html#copyright-and-license"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}